using System;
using System.Collections.Generic;
using System.Windows.Media;

namespace SciChartSimpleControl
{
    /// <summary>
    /// получатор цвета
    /// </summary>
    public class ColorsHelper
    {
        /// <summary>
        /// ChangeColorBrightness взято с http://www.pvladov.com
        /// Creates color with corrected brightness.
        /// </summary>
        /// <param name="color">Color to correct.</param>
        /// <param name="correctionFactor">The brightness correction factor. Must be between -1 and 1. 
        /// Negative values produce darker colors.</param>
        /// <returns>
        /// Corrected <see cref="Color"/> structure.
        /// </returns>
        private List<Color> UsedColors;
        public static Color ChangeColorBrightness(Color color, double correctionFactor)
        {
            double red =    (double)color.R;
            double green =  (double)color.G;
            double blue =   (double)color.B;

            if (correctionFactor < 0)
            {
                correctionFactor = 1 + correctionFactor;
                red *= correctionFactor;
                green *= correctionFactor;
                blue *= correctionFactor;
            }
            else
            {
                red = (255 - red) * correctionFactor + red;
                green = (255 - green) * correctionFactor + green;
                blue = (255 - blue) * correctionFactor + blue;
            }

            return Color.FromArgb(color.A, (byte)red, (byte)green, (byte)blue);
        }

        private Random _random;
        public ColorsHelper()
        {
            _random = new Random();
            UsedColors = new List<Color>();
        }
        /// <summary>
        /// сгенерировать рандомный цвет
        /// </summary>
        /// <returns></returns>
        public Color GetColor()
        {
            return GetPredefinedColor();
            //return GetRandomColor();
        }

        private Color GetRandomColor()
        {
            int red = 400;
            int green = 400;
            while (red > 200 && green > 200)
            {
                red = _random.Next(0, 255);
                green = _random.Next(0, 255);
            }
            int blue = _random.Next(0, Math.Min(255, 510 - red - green));


            var res = Color.FromRgb((byte)red, (byte)green, (byte)blue);
            return res;
        }

        private Color GetPredefinedColor()
        {
            int index = _random.Next(0, indexColorsTwo.Length - 1);
            Color color = (Color)ColorConverter.ConvertFromString(indexColorsTwo[index]);
            if (!UsedColors.Contains(color))
            {
                return color;
            }
            else
            {
                if (UsedColors.Count == indexColorsTwo.Length)
                {
                    return GetRandomColor();
                }
                return GetPredefinedColor();
            }
        }
        public Color GetColor(int index)
        {
            if ( index >= indexColorsTwo.Length || index <0)
            {
                return GetRandomColor();
            }
            Color color = (Color)ColorConverter.ConvertFromString(indexColorsTwo[index]);
            return color;
        }

        public static string[] indexColorsTwo = new string[]{"#0000FF","#FF0000","#00FF00","#006401","#010067",
            "#95003A","#007DB5","#FF00F6","#774D00","#0076FF","#FF937E","#71FFFE","#FFA6FE","#FFDB66","#6A826C","#FF029D","#FE8900","#7A4782","#7E2DD2","#85A900","#000000",
            "#FF0056","#A42400","#00AE7E","#683D3B","#BDC6FF","#263400","#BDD393","#00B917","#9E008E","#001544","#C28C9F","#FF74A3","#01D0FF","#004754","#E56FFE",
            "#788231","#0E4CA1","#91D0CB","#BE9970","#968AE8","#BB8800","#43002C","#00FFC6","#FFE502","#620E00","#008F9C","#7544B1","#B500FF",
            "#FF6E41","#005F39","#6B6882","#5FAD4E","#A75740", "#FFB167","#009BFF","#E85EBE"
        };
    }
}
