﻿using System;
using System.Collections.Generic;
using System.Linq;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;

namespace SciChartSimpleControl.SciChartSurfaceControl.Model
{

    public static class ViewModelsFactory
        {
            private static readonly Random Random = new Random(251916);
            private const int Count = 50;

        public static IRenderableSeriesViewModel New(SeriesTypes type, double valueShift, IDataSeries dataSeries = null)
        {
            var xyDataSeries = dataSeries ?? GetXyDataSeries(GetRanromWalk(valueShift));

            switch (type)
            {
                case SeriesTypes.Column:
                    {
                        return new ColumnRenderableSeriesViewModel { DataSeries = xyDataSeries, StyleKey = "ColumnStyle" };
                    }
                case SeriesTypes.Impulse:
                    {
                        return new ImpulseRenderableSeriesViewModel { DataSeries = xyDataSeries, StyleKey = "ImpulseStyle" };
                    }
                case SeriesTypes.Line:
                    {
                        return new LineRenderableSeriesViewModel { DataSeries = xyDataSeries, StyleKey = "LineStyle" };
                    }
                case SeriesTypes.Mountain:
                    {
                        return new MountainRenderableSeriesViewModel { DataSeries = xyDataSeries, StyleKey = "MountainStyle", };
                    }
                case SeriesTypes.XyScatter:
                    {
                        return new XyScatterRenderableSeriesViewModel { DataSeries = xyDataSeries, StyleKey = "ScatterStyle", DrawNaNAs = SciChart.Charting.Visuals.RenderableSeries.LineDrawMode.Gaps,ResamplingMode = SciChart.Data.Numerics.ResamplingMode.None }; //
                    }
            }
            throw new NotImplementedException("Unsupported Series Type");
        }

            // ReSharper disable PossibleMultipleEnumeration
            private static double[] GetRanromWalk(double valueShift)
            {
                var randomWalk = 1d;

                var yBuffer = new double[Count];
                for (var i = 0; i < Count; i++)
                {
                    randomWalk += (Random.NextDouble() - 0.498);
                    yBuffer[i] = randomWalk + valueShift;
                }

                return yBuffer;
            }

            private static IEnumerable<double> GetXValues(double[] data)
            {
                return Enumerable.Range(0, data.Length).Select(x => (double)x);
            }

            private static IXyDataSeries<double, double> GetXyDataSeries(double[] data)
            {
                var dataSeries = new XyDataSeries<double, double>();
                dataSeries.Append(GetXValues(data), data);

                return dataSeries;
            }
            // ReSharper restore PossibleMultipleEnumeration
        }
    }