﻿using SciChart.Charting.Common.Helpers;
using SciChart.Charting.Model.ChartSeries;
using SciChart.Charting.Model.DataSeries;
using SciChart.Charting.ViewportManagers;
using SciChart.Core.Extensions;

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Timers;
using System.Windows.Media;
using SciChartSimpleControl.SciChartSurfaceControl.Model;
using SciChartSimpleControl.SciChartSurfaceControl.ViewModel;
using TikFramework.Command;
using TikFramework.ViewModel;

namespace SciChartSimpleControl.SciChartSurfaceControl.ViewModel
{
    public enum ChartTypes
    {
        Array,
        History,
        Radial
    }
    public class SciChartSurfaceViewModel : BaseViewModel
    {
        private readonly IViewportManager _viewportManager = new DefaultViewportManager();

        private SeriesTypes _selectedSeriesType;
        private bool _ZoomEnabled;
        public bool ZoomEnabled { get { return _ZoomEnabled; } set { if (_ZoomEnabled == value) return; Set(ref _ZoomEnabled, value); PanEnabled = !value; } }
        private bool _PanEnabled;
        public bool PanEnabled { get { return _PanEnabled; } set { if (_PanEnabled == value) return; Set(ref _PanEnabled, value); ZoomEnabled = !value; } }
        private bool _MouseWheelEnabled;
        public bool MouseWheelEnabled { get { return _MouseWheelEnabled; } set { if (_MouseWheelEnabled == value) return;Set(ref _MouseWheelEnabled, value);}}

        private bool _RollOverEnabled;
        public bool RollOverEnabled { get { return _RollOverEnabled; } set { Set(ref _RollOverEnabled, value); } }

        private bool _SelectionModifierEnabled;
        public bool SelectionModifierEnabled { get { return _SelectionModifierEnabled; } set { Set(ref _SelectionModifierEnabled, value); } }

        private string _ChartTitle;
        public string ChartTitle { get { return _ChartTitle; } set { Set(ref _ChartTitle, value); } }

        private SciChart.Charting.Visuals.Axes.AutoRange _AutoRangeMode;
        public SciChart.Charting.Visuals.Axes.AutoRange AutoRangeMode { get { return _AutoRangeMode; } set { Set(ref _AutoRangeMode, value); UpdateXAxesAutoRangeProperty(); } }
        private ColorsHelper ColorsHelperInstance;

        // TODO: разобраться как забиндить, а то не обновляется
        private void UpdateXAxesAutoRangeProperty()
        {
            foreach (IAxisViewModel axis in XAxes)
            {
                axis.AutoRange = AutoRangeMode;
            }
        }

        private ChartTypes _ControlChartType;
        public ChartTypes ControlChartType { get { return _ControlChartType; } set { Set(ref _ControlChartType, value); UpdateXAxisType(); } }

        private void UpdateXAxisType()
        {
            // пока оставляем одну ось, при необходимости тут доделать
            XAxes.Clear();
            IAxisViewModel XAxisVM;
            switch (ControlChartType)
            {
                case ChartTypes.History:
                    {
                        XAxisVM = new DateTimeAxisViewModel() { Id = "DefaultAxisId" };
                        XAxisVM.AutoRange = AutoRangeMode;
                        break;
                        //Id = "HistoricalAxis"
                    }
                default:
                    {
                        XAxisVM = new NumericAxisViewModel() { Id = "DefaultAxisId" };
                        XAxisVM.AutoRange = AutoRangeMode;
                        break;
                        //Id = "ArraysAxis"
                        // TODO: разобраться как забиндить, а то не обновляется
                        //Binding binding = new Binding();
                        //binding.Path = new PropertyPath("AutoRange");
                        //binding.Source = AutoRangeMode;  // view model?
                        //BindingOperations.SetBinding(XAxisVM, AxisCore.AutoRangeProperty , binding);
                    }
            }
            XAxes.Add(XAxisVM);
        }

        public List<DateTime> GetSlicesDates()
        {
            if ( ControlChartType!= ChartTypes.History)
            {
                return null;
            }
            List<DateTime> result = AnnotationsCollection.Select(x => (DateTime)x.X1).ToList();
            return result;
        }

        private RaiseCommand _addCommand;
        private readonly ActionCommand _removeCommand;
        private RaiseCommand _clearCommand;
        private RaiseCommand _RedrawByPiecesCommand;
        private RaiseCommand _StopRedrawByPiecesCommand;
        private RaiseCommand _zoomExtentsCommand;
        private RaiseCommand _PlayPauseRedrawByPiecesCommand;
        private readonly ActionCommand _selectionChangedCommand;
        public ObservableCollection<IAxisViewModel> XAxes { get; set; }

        private readonly ObservableCollection<IRenderableSeriesViewModel> _renderableSeriesViewModels;

        public ObservableCollection<IRenderableSeriesViewModel> RenderableSeriesViewModels
        {
            get { return _renderableSeriesViewModels; }
        }
        private List<DataSeriesWithTypeViewModel> DataSeries {get; set; }
        private List<DataSeriesWithTypeViewModel> AdditionalDataSeries { get; set; }
        private bool _running;
        private ObservableCollection<IAnnotationViewModel> _AnnotationsCollection; 
        public ObservableCollection<IAnnotationViewModel> AnnotationsCollection { get { return _AnnotationsCollection; } set { Set(ref _AnnotationsCollection, value);  } }
        private ObservableCollection<IAnnotationViewModel> _VerticalSlicesCollection;
        public ObservableCollection<IAnnotationViewModel> VerticalSlicesCollection { get { return _VerticalSlicesCollection; } set { Set(ref _VerticalSlicesCollection, value); } }

        public SciChartSurfaceViewModel()
        {
            _renderableSeriesViewModels = new ObservableCollection<IRenderableSeriesViewModel>();
            ValuesCollection = new ObservableCollection<WrappedValuesViewModel>();
            ValuesCollection.CollectionChanged += ValuesCollection_CollectionChanged;
            XAxes = new ObservableCollection<IAxisViewModel>();
            AnnotationsCollection = new ObservableCollection<IAnnotationViewModel>();
            VerticalSlicesCollection = new ObservableCollection<IAnnotationViewModel>();
            DataSeries = new List<DataSeriesWithTypeViewModel>();
            AdditionalDataSeries = new List<DataSeriesWithTypeViewModel>();
            AddCommands();
            ZoomEnabled = true;
            MouseWheelEnabled = true;
            RollOverEnabled = false;
            SelectionModifierEnabled = true;
            AutoRangeMode = SciChart.Charting.Visuals.Axes.AutoRange.Once;
            ControlChartType = ChartTypes.Array;
            ColorsHelperInstance = new ColorsHelper();
            // TODO: переделать на RaiseCommand
            _removeCommand = new ActionCommand(() =>
            {
                RenderableSeriesViewModels.RemoveWhere(s => s.IsSelected);
            }, () => RenderableSeriesViewModels.Any(s => s.IsSelected));

            _selectionChangedCommand = new ActionCommand(() =>
            {
                var rSeriesVm = RenderableSeriesViewModels.FirstOrDefault(s => s.IsSelected);
                if (rSeriesVm != null)
                {
                    //SelectedSeriesType = SeriesTypes.FirstOrDefault(x => x.Type == rSeriesVm.GetType());
                }
                RemoveCommand.RaiseCanExecuteChanged();
            });
            //FillSeriesTypes();
            SelectedSeriesType = SeriesTypes.Line;
            RemoveCommand.RaiseCanExecuteChanged();
        }
        private void ValuesCollection_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.NewItems != null)
            {
                foreach (WrappedValuesViewModel newItem in e.NewItems)
                {
                    //Add listener for each item on PropertyChanged event
                    newItem.PropertyChanged += this.OnItemPropertyChanged;
                }
            }

            if (e.OldItems != null)
            {
                foreach (WrappedValuesViewModel oldItem in e.OldItems)
                {
                    oldItem.PropertyChanged -= this.OnItemPropertyChanged;
                }
            }

            UpdateDataSeries();
        }
        void OnItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            var name = e.PropertyName;
            if (e.PropertyName == "ConvertedValue" || e.PropertyName == "ConvertedValues")
            {
                UpdateDataSeries();
            }
        }
        private void UpdateDataSeries()
        {
            DataSeries.Clear();
            AdditionalDataSeries.Clear();
            RenderableSeriesViewModels.Clear(); // Надо ли?
            // привести в соответствие VM со свойствами и VM серий данных

            // Разделить на типы графиков
            switch (ControlChartType)
            {
                case ChartTypes.Array:
                    {
                        FillArraySeries();
                        break;
                    }
                case ChartTypes.History:
                    {
                        FillHistorySeries();
                        break;
                    }
                case ChartTypes.Radial:
                    {
                        FillRadialSeries();
                        break;
                    }
            }
            UpdateRenderableSeries(); // TEST
            UpdateAdditionalRenderableSeries();
        }
        private void UpdateRenderableSeries()
        {
            RenderableSeriesViewModels.Clear();
            foreach (DataSeriesWithTypeViewModel series in DataSeries)
            {
                //var renderableSeries = ViewModelsFactory.New(series.SeriesType, 0, series.DataSeries);
                RenderableSeriesViewModels.Add(series.RenderableSeries);
            }
            ZoomExtents();
        }
        private void UpdateAdditionalRenderableSeries()
        {
            foreach (DataSeriesWithTypeViewModel seriesWithType in AdditionalDataSeries)
            {
                //var renderableSeries = ViewModelsFactory.New(seriesWithType.SeriesType, 0, seriesWithType.DataSeries);
                //RenderableSeriesViewModels.Add( renderableSeries);
                RenderableSeriesViewModels.Add(seriesWithType.RenderableSeries);
            }
            ZoomExtents();
        }

        private void FillRadialSeries()
        {
            if ( ValuesCollection.Count != 2 || ValuesCollection[0].RawValue ==null || ValuesCollection[1].RawValue == null)
            {
                return;
            }

            double[] arrX = ValuesCollection[0].RawValue;
            double[] arrY = ValuesCollection[1].RawValue;
            if (arrX.Length != arrY.Length)
            {
                int minLen = Math.Min(arrX.Length, arrY.Length);
                arrX = arrX.Take(minLen).ToArray();
                arrY = arrY.Take(minLen).ToArray();
            }
            XyDataSeries<double, double> series = new XyDataSeries<double, double>();
            series.AcceptsUnsortedData = true;
            series.Append(arrX, arrY);
            DataSeries.Add(new DataSeriesWithTypeViewModel(series,ControlChartType, SelectedSeriesType, ColorsHelperInstance.GetColor(DataSeries.Count)));
        }
        private void FillHistorySeries()
        {
            foreach (WrappedValuesViewModel valueVM in ValuesCollection)
            {
                if (valueVM.HistoricalValues != null && valueVM.HistoricalValues.Count != 0)
                {
                    XyDataSeries<DateTime, double> series = new XyDataSeries<DateTime, double>();
                    series.Append(valueVM.HistoricalValues.Select(x=>x.Date) , valueVM.HistoricalValues.Select(x => x.Value));
                    var dataSeriesVM = new DataSeriesWithTypeViewModel(series, ControlChartType, SelectedSeriesType, ColorsHelperInstance.GetColor(DataSeries.Count));
                    dataSeriesVM.RenderableSeries.DataSeries.SeriesName = valueVM.SeriesName;
                    DataSeries.Add(dataSeriesVM);
                }
            }
        }
        private void FillArraySeries()
        {
            foreach (WrappedValuesViewModel valueVM in ValuesCollection)
            {
                if (valueVM.RawValue != null && valueVM.RawValue.Length!=0)
                {
                    // upd: используем конвертированное значение
                    XyDataSeries<double, double> series = new XyDataSeries<double, double>();
                    var xw = Enumerable.Range(0, valueVM.RawValue.Length).Select(x => (double)x);
                    series.Append(xw, valueVM.RawValue);
                    var dataSeriesVM = new DataSeriesWithTypeViewModel(series, ControlChartType, SelectedSeriesType, ColorsHelperInstance.GetColor(DataSeries.Count));
                    dataSeriesVM.RenderableSeries.DataSeries.SeriesName = valueVM.SeriesName;
                    DataSeries.Add(dataSeriesVM);
                }
            }
        }

        #region Commands
        private void AddCommands()
        {
            AddCommand = new RaiseCommand(AddCommandExecute, AddCommandCanExecute);
            ZoomExtentsCommand = new RaiseCommand(ZoomExtentsCommandExecute, ClearCommandCanExecute);
            ClearCommand = new RaiseCommand(ClearCommandExecute, ClearCommandCanExecute);
            AddSliceCommand = new RaiseCommand(AddSliceCommandExecute, ClearCommandCanExecute);
            RemoveSliceCommand = new RaiseCommand(RemoveSliceCommandExecute, ClearCommandCanExecute);
        }
        private void AddSliceCommandExecute(object param)
        {
            var slice = new VerticalLineAnnotationViewModel()
            {
                //YAxisId = MainChartSurface.YAxes[0].Id,
                //XAxisId = axisId,
                StyleKey = "sliceStyle",
            };

            switch (ControlChartType)
            {
                case ChartTypes.History:
                    {
                        slice.X1 = (DateTime)param;
                        break;
                    }
                default:
                    {
                        slice.X1 = (double)param;
                        break;
                    }
            }

            //double xPointValue = (double)param;

            //string axisId = XAxes[0].Id;
            
            //VerticalSlicesCollection.Add(slice);
            AnnotationsCollection.Add(slice);
        }
        private void RemoveSliceCommandExecute(object param)
        {
            //List<IAnnotationViewModel> annotationsToRemove = new List<IAnnotationViewModel>();
            //foreach (IAnnotationViewModel annotation in AnnotationsCollection)
            //{
            //    if (annotation.IsSelected)
            //    {
            //        annotationsToRemove.Add(annotation);
            //    }
            //}
            AnnotationsCollection.RemoveWhere(annotation => annotation.IsSelected);
        }
        private void AddCommandExecute(object param)
        {
            RenderableSeriesViewModels.Add(ViewModelsFactory.New(SelectedSeriesType, 0, param as IDataSeries));
        }
        private bool AddCommandCanExecute(object param)
        {
            return SelectedSeriesType != null;
        }
        private void ZoomExtentsCommandExecute(object param)
        {
            ZoomExtents();
        }
        private void ClearCommandExecute(object param)
        {
            RenderableSeriesViewModels.Clear();
            AnnotationsCollection.Clear();
        }
        private bool ClearCommandCanExecute(object param)
        {
            return RenderableSeriesViewModels.Count > 0;
        }
        public IViewportManager ViewportManager
        {
            get { return _viewportManager; }
        }

        //public List<SeriesType> SeriesTypes
        //{
        //    get { return _seriesTypes; }
        //}

        public SeriesTypes SelectedSeriesType
        {
            get { return _selectedSeriesType; }
            set
            {
                if (_selectedSeriesType != value)
                {
                    _selectedSeriesType = value;
                    ChangeRenderableSeriesType();
                    OnPropertyChanged("SelectedSeriesType");
                }
            }
        }
        public RaiseCommand AddCommand
        {
            get { return _addCommand;} private set { _addCommand = value; }
        }
        public RaiseCommand ZoomExtentsCommand
        {
            get { return _zoomExtentsCommand; }
            private set { _zoomExtentsCommand = value; }
        }
        public RaiseCommand ClearCommand
        {
            get { return _clearCommand; }
            private set { _clearCommand = value; }
        }
        public RaiseCommand RedrawByPiecesCommand
        {
            get { return _RedrawByPiecesCommand; }
            private set { _RedrawByPiecesCommand = value; }
        }
        public RaiseCommand StopRedrawByPiecesCommand
        {
            get { return _StopRedrawByPiecesCommand; }
            private set { _StopRedrawByPiecesCommand = value; }
        }
        public RaiseCommand PlayPauseRedrawByPiecesCommand
        {
            get { return _PlayPauseRedrawByPiecesCommand; }
            private set { _PlayPauseRedrawByPiecesCommand = value; }
        }
        public ActionCommand RemoveCommand
        {
            get { return _removeCommand; }
        }
        public ActionCommand SelectionChangedCommand
        {
            get { return _selectionChangedCommand; }
        }
        public RaiseCommand AddSliceCommand { get; internal set; }
        public RaiseCommand RemoveSliceCommand { get; internal set; }
        public ObservableCollection<WrappedValuesViewModel> ValuesCollection { get; set; }
        #endregion
        private void ChangeRenderableSeriesType()
        {
            var rSeriesVm = RenderableSeriesViewModels.FirstOrDefault(s => s.IsSelected);
            if (rSeriesVm != null)
            {
                var index = RenderableSeriesViewModels.IndexOf(rSeriesVm);

                RenderableSeriesViewModels[index] = ViewModelsFactory.New(SelectedSeriesType, 0, RenderableSeriesViewModels[index].DataSeries);
                RenderableSeriesViewModels[index].IsSelected = true;

                ZoomExtents();
            }
        }
        private void ZoomExtents()
        {
            ViewportManager.BeginInvoke(() => { ViewportManager.ZoomExtents(); }); //ViewportManager.ZoomExtents();// AnimateZoomExtents(TimeSpan.FromMilliseconds(500));
        }

        //private void FillSeriesTypes()
        //{
        //    _seriesTypes = new List<SeriesType>
        //{
        //    new SeriesType(typeof (ColumnRenderableSeriesViewModel), "IconColumn"),
        //    new SeriesType(typeof (ImpulseRenderableSeriesViewModel), "IconImpulse"),
        //    new SeriesType(typeof (LineRenderableSeriesViewModel), "IconLine"),
        //    new SeriesType(typeof (MountainRenderableSeriesViewModel), "IconMountain"),
        //    new SeriesType(typeof (XyScatterRenderableSeriesViewModel), "IconScatter"),
        //};
        //}
    }
}